import math


def extract_client_cpu_limit(benchmark_config):
    db_cpu_limit = benchmark_config["clientconfig"]["resources"]["requests"]["cpus"]
    ceil_db_cpu_limit = math.ceil(float(db_cpu_limit))
    return ceil_db_cpu_limit


def extract_client_container_image(benchmark_config):
    client_container_image = None
    if "clientconfig" in benchmark_config:
        if "run_image" in benchmark_config["clientconfig"]:
            client_container_image = benchmark_config["clientconfig"]["run_image"]
    return client_container_image



#  Apache License Version 2.0
#
#  Copyright (c) 2021., Redis Labs
#  All rights reserved.
#

import logging
from urllib.request import urlopen
from urllib.error import URLError
import logging.handlers
from flask import Flask, jsonify, request
from marshmallow import Schema, fields, ValidationError
from redis_benchmarks_specification import __version__
from json import dumps, loads
import redis
import argparse
from flask_httpauth import HTTPBasicAuth

from redis_benchmarks_specification.__common__.env import (
    STREAM_KEYNAME_GH_EVENTS_COMMIT,
    GH_REDIS_SERVER_HOST,
    GH_REDIS_SERVER_PORT,
    GH_REDIS_SERVER_AUTH,
    REDIS_AUTH_SERVER_HOST,
    REDIS_AUTH_SERVER_PORT,
    LOG_FORMAT,
    LOG_DATEFMT,
    LOG_LEVEL,
)

auth = HTTPBasicAuth()

app = Flask(__name__)
conn = None


@auth.verify_password
def verify_password(username, password):
    result = False
    try:
        auth_server_conn = redis.StrictRedis(
            host=REDIS_AUTH_SERVER_HOST,
            port=REDIS_AUTH_SERVER_PORT,
            decode_responses=True,
            username=username,
            password=password,
        )
        auth_server_conn.ping()
        result = True
    except redis.exceptions.ResponseError:
        result = False
    return result


def commit_schema_to_stream(json_str: str):
    """ uses to the provided JSON dict of fields and pushes that info to the corresponding stream  """
    fields = loads(json_str)
    reply_fields = loads(json_str)
    result = False
    error_msg = None
    github_url = "https://github.com/redis/redis/archive/{}.zip".format(
        fields["git_hash"]
    )
    try:
        response = urlopen(github_url, timeout=5)
        content = response.read()
        fields["zip_archive"] = bytes(content)
        fields["zip_archive_len"] = len(bytes(content))
        reply_fields["archived_zip"] = True
        result = True
    except URLError as e:
        error_msg = "Catched URLError while fetching {} content. Error {}".format(
            github_url, e.__str__()
        )
        logging.error(error_msg)
        result = False

    if result is True:
        id = conn.xadd(STREAM_KEYNAME_GH_EVENTS_COMMIT, fields)
        reply_fields["id"] = id

    return result, reply_fields, error_msg


class CommitSchema(Schema):
    git_branch = fields.String(required=False)
    git_hash = fields.String(required=True)


@app.route("/api/gh/redis/redis/commits", methods=["POST"])
@auth.login_required
def base():
    # Get Request body from JSON
    request_data = request.json
    schema = CommitSchema()
    try:
        # Validate request body against schema data types
        result = schema.load(request_data)
    except ValidationError as err:
        # Return a nice message if validation fails
        return jsonify(err.messages), 400

    # Convert request body back to JSON str
    data_now_json_str = dumps(result)

    result, response_data, err_message = commit_schema_to_stream(data_now_json_str)
    if result is False:
        return jsonify(err_message), 400

    # Send data back as JSON
    return jsonify(response_data), 200


def main():
    global conn
    parser = argparse.ArgumentParser(
        description="redis-benchmarks-specification API",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    logging.info("redis-benchmarks-specification API. v{}".format(__version__))
    parser.add_argument(
        "--logname", type=str, default=None, help="logname to write the logs to"
    )
    args = parser.parse_args()
    print(
        "Using redis available at: {}:{} for event store.".format(
            GH_REDIS_SERVER_HOST, GH_REDIS_SERVER_PORT
        )
    )
    print(
        "Using redis available at: {}:{} as auth server.".format(
            REDIS_AUTH_SERVER_HOST, REDIS_AUTH_SERVER_PORT
        )
    )

    if args.logname is not None:
        print("Writting log to {}".format(args.logname))
        handler = logging.handlers.RotatingFileHandler(
            args.logname, maxBytes=1024 * 1024
        )
        logging.getLogger("werkzeug").setLevel(logging.DEBUG)
        logging.getLogger("werkzeug").addHandler(handler)
        app.logger.setLevel(LOG_LEVEL)
        app.logger.addHandler(handler)
    else:
        # logging settings
        logging.basicConfig(
            format=LOG_FORMAT,
            level=LOG_LEVEL,
            datefmt=LOG_DATEFMT,
        )
    logging.info(
        "Using redis available at: {}:{}".format(
            GH_REDIS_SERVER_HOST, GH_REDIS_SERVER_PORT
        )
    )
    conn = redis.StrictRedis(
        host=GH_REDIS_SERVER_HOST,
        port=GH_REDIS_SERVER_PORT,
        decode_responses=True,
        password=GH_REDIS_SERVER_AUTH,
    )

    app.run(host="0.0.0.0")


if __name__ == "__main__":
    main()



import argparse

from redis_benchmarks_specification.__common__.env import (
    MACHINE_CPU_COUNT,
    SPECS_PATH_SETUPS,
    SPECS_PATH_TEST_SUITES,
    DATASINK_RTS_HOST,
    DATASINK_RTS_PORT,
    DATASINK_RTS_AUTH,
    DATASINK_RTS_USER,
    DATASINK_RTS_PUSH,
)


def create_self_contained_coordinator_args():
    parser = argparse.ArgumentParser(
        description="redis-benchmarks-spec runner(self-contained)",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "--cpu-count",
        type=int,
        default=MACHINE_CPU_COUNT,
        help="Specify how much of the available CPU resources the coordinator can use.",
    )
    parser.add_argument(
        "--logname", type=str, default=None, help="logname to write the logs to"
    )
    parser.add_argument(
        "--consumer-start-id",
        type=str,
        default=">",
    )
    parser.add_argument(
        "--setups-folder",
        type=str,
        default=SPECS_PATH_SETUPS,
        help="Setups folder, containing the build environment variations sub-folder that we use to trigger different build artifacts",
    )
    parser.add_argument(
        "--test-suites-folder",
        type=str,
        default=SPECS_PATH_TEST_SUITES,
        help="Test suites folder, containing the different test variations",
    )
    parser.add_argument(
        "--datasink_redistimeseries_host", type=str, default=DATASINK_RTS_HOST
    )
    parser.add_argument(
        "--datasink_redistimeseries_port", type=int, default=DATASINK_RTS_PORT
    )
    parser.add_argument(
        "--datasink_redistimeseries_pass", type=str, default=DATASINK_RTS_AUTH
    )
    parser.add_argument(
        "--datasink_redistimeseries_user", type=str, default=DATASINK_RTS_USER
    )
    parser.add_argument(
        "--datasink_push_results_redistimeseries",
        default=DATASINK_RTS_PUSH,
        action="store_true",
        help="uploads the results to RedisTimeSeries. Proper credentials are required",
    )
    return parser



import argparse
import io
import logging
import tempfile
import shutil
import redis
import os
from zipfile import ZipFile, ZipInfo

from redis_benchmarks_specification.__builder__.schema import get_build_config
from redis_benchmarks_specification.__common__.env import (
    STREAM_KEYNAME_GH_EVENTS_COMMIT,
    GH_REDIS_SERVER_HOST,
    GH_REDIS_SERVER_PORT,
    GH_REDIS_SERVER_AUTH,
    LOG_FORMAT,
    LOG_DATEFMT,
    LOG_LEVEL,
    SPECS_PATH_SETUPS,
    STREAM_GH_EVENTS_COMMIT_BUILDERS_CG,
    STREAM_KEYNAME_NEW_BUILD_EVENTS,
)
from redis_benchmarks_specification.__common__.package import PACKAGE_DIR


class ZipFileWithPermissions(ZipFile):
    def _extract_member(self, member, targetpath, pwd):
        if not isinstance(member, ZipInfo):
            member = self.getinfo(member)

        targetpath = super()._extract_member(member, targetpath, pwd)

        attr = member.external_attr >> 16
        if attr != 0:
            os.chmod(targetpath, attr)
        return targetpath


def main():
    parser = argparse.ArgumentParser(
        description="redis-benchmarks-spec builder",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "--logname", type=str, default=None, help="logname to write the logs to"
    )
    parser.add_argument(
        "--setups-folder",
        type=str,
        default=SPECS_PATH_SETUPS,
        help="Setups folder, containing the build environment variations sub-folder that we use to trigger different build artifacts",
    )
    parser.add_argument(
        "--consumer-start-id",
        type=str,
        default=">",
    )
    args = parser.parse_args()

    if args.logname is not None:
        print("Writting log to {}".format(args.logname))
        logging.basicConfig(
            filename=args.logname,
            filemode="a",
            format=LOG_FORMAT,
            datefmt=LOG_DATEFMT,
            level=LOG_LEVEL,
        )
    else:
        # logging settings
        logging.basicConfig(
            format=LOG_FORMAT,
            level=LOG_LEVEL,
            datefmt=LOG_DATEFMT,
        )
    logging.info("Using package dir {} for inner file paths".format(PACKAGE_DIR))
    builders_folder = os.path.abspath(
        PACKAGE_DIR + "/" + args.setups_folder + "/builders"
    )
    different_build_specs = os.listdir(builders_folder)
    logging.info(
        "Using the following build specs folder {}, containing {} different specs.".format(
            builders_folder, len(different_build_specs)
        )
    )

    logging.info(
        "Using redis available at: {}:{} to read the event streams".format(
            GH_REDIS_SERVER_HOST, GH_REDIS_SERVER_PORT
        )
    )
    try:
        conn = redis.StrictRedis(
            host=GH_REDIS_SERVER_HOST,
            port=GH_REDIS_SERVER_PORT,
            decode_responses=False,  # dont decode due to zip archive
            password=GH_REDIS_SERVER_AUTH,
        )
        conn.ping()
    except redis.exceptions.ConnectionError as e:
        logging.error(
            "Unable to connect to redis available at: {}:{} to read the event streams".format(
                GH_REDIS_SERVER_HOST, GH_REDIS_SERVER_PORT
            )
        )
        logging.error("Error message {}".format(e.__str__()))
        exit(1)

    logging.info("checking build spec requirements")
    already_checked_images = []
    for build_spec in different_build_specs:
        build_config, id = get_build_config(builders_folder + "/" + build_spec)
        if build_config["kind"] == "docker":
            build_image = build_config["build_image"]
            if build_image not in already_checked_images:
                logging.info(
                    "Build {} requirement: checking build image {} is available.".format(
                        id, build_image
                    )
                )
                import docker

                client = docker.from_env()
                image = client.images.pull(build_image)
                logging.info(
                    "Build {} requirement: build image {} is available with id: {}.".format(
                        id, build_image, image.id
                    )
                )
                already_checked_images.append(build_image)
            else:
                logging.info(
                    "Build {} requirement: build image {} availability was already checked.".format(
                        id, build_image
                    )
                )

    try:
        conn.xgroup_create(
            STREAM_KEYNAME_GH_EVENTS_COMMIT,
            STREAM_GH_EVENTS_COMMIT_BUILDERS_CG,
            mkstream=True,
        )
        logging.info(
            "Created consumer group named {} to distribute work.".format(
                STREAM_GH_EVENTS_COMMIT_BUILDERS_CG
            )
        )
    except redis.exceptions.ResponseError:
        logging.info(
            "Consumer group named {} already existed.".format(
                STREAM_GH_EVENTS_COMMIT_BUILDERS_CG
            )
        )
    previous_id = None
    while True:
        if previous_id is None:
            previous_id = args.consumer_start_id
        logging.info("Entering blocking read waiting for work.")
        newTestInfo = conn.xreadgroup(
            STREAM_GH_EVENTS_COMMIT_BUILDERS_CG,
            "{}-proc#{}".format(STREAM_GH_EVENTS_COMMIT_BUILDERS_CG, "1"),
            {STREAM_KEYNAME_GH_EVENTS_COMMIT: previous_id},
            count=1,
            block=0,
        )
        if len(newTestInfo[0]) < 2 or len(newTestInfo[0][1]) < 1:
            previous_id = ">"
            continue
        streamId, testDetails = newTestInfo[0][1][0]
        logging.info("Received work . Stream id {}.".format(streamId))
        # commit = None
        # commited_date = ""
        # tag = ""
        docker_client = docker.from_env()
        from pathlib import Path

        home = str(Path.home())
        if b"git_hash" in testDetails:
            git_hash = testDetails[b"git_hash"]
            logging.info("Received commit hash specifier {}.".format(git_hash))
            buffer = testDetails[b"zip_archive"]

            for build_spec in different_build_specs:
                build_config, id = get_build_config(builders_folder + "/" + build_spec)
                build_image = build_config["build_image"]
                run_image = build_image
                if "run_image" in build_config:
                    run_image = build_config["run_image"]
                compiler = build_config["compiler"]
                cpp_compiler = build_config["cpp_compiler"]
                build_os = build_config["os"]
                build_arch = build_config["arch"]

                build_artifacts = ["redis-server"]
                if "build_artifacts" in build_config:
                    build_artifacts = build_config["build_artifacts"]
                build_vars_str = ""
                if "env" in build_config:
                    if build_config["env"] is not None:
                        for k, v in build_config["env"].items():
                            build_vars_str += '{}="{}" '.format(k, v)

                temporary_dir = tempfile.mkdtemp(dir=home)
                logging.info(
                    "Using local temporary dir to persist redis build artifacts. Path: {}".format(
                        temporary_dir
                    )
                )
                z = ZipFileWithPermissions(io.BytesIO(buffer))
                z.extractall(temporary_dir)
                redis_dir = os.listdir(temporary_dir + "/")[0]
                redis_temporary_dir = temporary_dir + "/" + redis_dir + "/"
                logging.info("Using redis temporary dir {}".format(redis_temporary_dir))
                build_command = 'bash -c "make Makefile.dep {} && cd ./deps && CXX={} CC={} make {} {} -j && cd .. && CXX={} CC={} make {} {} -j"'.format(
                    build_vars_str,
                    cpp_compiler,
                    compiler,
                    " ".join(
                        [
                            "hdr_histogram",
                            "hiredis",
                            "jemalloc",
                            "linenoise",
                            "lua",
                        ]
                    ),
                    build_vars_str,
                    cpp_compiler,
                    compiler,
                    "redis-server",
                    build_vars_str,
                )
                logging.info(
                    "Using the following build command {}".format(build_command)
                )
                docker_client.containers.run(
                    image=build_image,
                    volumes={
                        redis_temporary_dir: {"bind": "/mnt/redis/", "mode": "rw"},
                    },
                    auto_remove=True,
                    privileged=True,
                    working_dir="/mnt/redis/",
                    command=build_command,
                )
                build_stream_fields = {
                    "id": id,
                    "git_hash": git_hash,
                    "build_image": build_image,
                    "run_image": run_image,
                    "compiler": compiler,
                    "cpp_compiler": cpp_compiler,
                    "os": build_os,
                    "arch": build_arch,
                    "build_vars": build_vars_str,
                    "build_command": build_command,
                    "build_artifacts": ",".join(build_artifacts),
                }
                for artifact in build_artifacts:
                    bin_artifact = open(
                        "{}src/{}".format(redis_temporary_dir, artifact), "rb"
                    ).read()
                    build_stream_fields[artifact] = bytes(bin_artifact)
                    build_stream_fields["{}_len_bytes".format(artifact)] = len(
                        bytes(bin_artifact)
                    )
                result = True
                if result is True:
                    stream_id = conn.xadd(
                        STREAM_KEYNAME_NEW_BUILD_EVENTS, build_stream_fields
                    )
                    logging.info(
                        "sucessfully built build variant {} for redis git_sha {}. Stream id: {}".format(
                            id, git_hash, stream_id
                        )
                    )
                shutil.rmtree(temporary_dir, ignore_errors=True)

        else:
            logging.error("Missing commit information within received message.")
            continue



import json
import logging
import math
import pathlib
import sys
import tempfile
import shutil
import traceback
import docker
import redis
import os
from pathlib import Path

from redisbench_admin.run.common import (
    get_start_time_vars,
    prepare_benchmark_parameters,
)
from redisbench_admin.utils.benchmark_config import (
    extract_redis_dbconfig_parameters,
    get_final_benchmark_config,
)
from redisbench_admin.utils.local import get_local_run_full_filename
from redisbench_admin.utils.results import post_process_benchmark_results

from redis_benchmarks_specification.__common__.env import (
    GH_REDIS_SERVER_HOST,
    GH_REDIS_SERVER_PORT,
    GH_REDIS_SERVER_AUTH,
    LOG_FORMAT,
    LOG_DATEFMT,
    LOG_LEVEL,
    STREAM_KEYNAME_NEW_BUILD_EVENTS,
    GH_REDIS_SERVER_USER,
    STREAM_GH_NEW_BUILD_RUNNERS_CG,
)
from redis_benchmarks_specification.__common__.spec import (
    extract_client_cpu_limit,
    extract_client_container_image,
)
from redis_benchmarks_specification.__self_contained_coordinator__.args import (
    create_self_contained_coordinator_args,
)
from redis_benchmarks_specification.__setups__.topologies import get_topologies


def main():
    parser = create_self_contained_coordinator_args()
    args = parser.parse_args()

    if args.logname is not None:
        print("Writting log to {}".format(args.logname))
        logging.basicConfig(
            filename=args.logname,
            filemode="a",
            format=LOG_FORMAT,
            datefmt=LOG_DATEFMT,
            level=LOG_LEVEL,
        )
    else:
        # logging settings
        logging.basicConfig(
            format=LOG_FORMAT,
            level=LOG_LEVEL,
            datefmt=LOG_DATEFMT,
        )
    topologies_folder = os.path.abspath(args.setups_folder + "/topologies")
    logging.info("Using topologies folder dir {}".format(topologies_folder))
    topologies_files = pathlib.Path(topologies_folder).glob("*.yml")
    topologies_files = [str(x) for x in topologies_files]
    logging.info(
        "Reading topologies specifications from: {}".format(
            " ".join([str(x) for x in topologies_files])
        )
    )
    topologies_map = get_topologies(topologies_files[0])
    testsuites_folder = os.path.abspath(args.test_suites_folder)
    logging.info("Using test-suites folder dir {}".format(testsuites_folder))
    logging.info(
        "Using redis available at: {}:{} to read the event streams".format(
            GH_REDIS_SERVER_HOST, GH_REDIS_SERVER_PORT
        )
    )
    try:
        conn = redis.StrictRedis(
            host=GH_REDIS_SERVER_HOST,
            port=GH_REDIS_SERVER_PORT,
            decode_responses=False,  # dont decode due to binary archives
            password=GH_REDIS_SERVER_AUTH,
            username=GH_REDIS_SERVER_USER,
        )
        conn.ping()
    except redis.exceptions.ConnectionError as e:
        logging.error(
            "Unable to connect to redis available at: {}:{} to read the event streams".format(
                GH_REDIS_SERVER_HOST, GH_REDIS_SERVER_PORT
            )
        )
        logging.error("Error message {}".format(e.__str__()))
        exit(1)

    logging.info("checking build spec requirements")
    try:
        conn.xgroup_create(
            STREAM_KEYNAME_NEW_BUILD_EVENTS,
            STREAM_GH_NEW_BUILD_RUNNERS_CG,
            mkstream=True,
        )
        logging.info(
            "Created consumer group named {} to distribute work.".format(
                STREAM_GH_NEW_BUILD_RUNNERS_CG
            )
        )
    except redis.exceptions.ResponseError:
        logging.info(
            "Consumer group named {} already existed.".format(
                STREAM_GH_NEW_BUILD_RUNNERS_CG
            )
        )
    previous_id = None
    docker_client = docker.from_env()
    home = str(Path.home())
    # TODO: confirm we do have enough cores to run the spec
    # availabe_cpus = args.cpu_count

    while True:
        logging.info("Entering blocking read waiting for work.")
        if previous_id is None:
            previous_id = args.consumer_start_id
        newTestInfo = conn.xreadgroup(
            STREAM_GH_NEW_BUILD_RUNNERS_CG,
            "{}-self-contained-proc#{}".format(STREAM_GH_NEW_BUILD_RUNNERS_CG, "1"),
            {STREAM_KEYNAME_NEW_BUILD_EVENTS: previous_id},
            count=1,
            block=0,
        )
        if len(newTestInfo[0]) < 2 or len(newTestInfo[0][1]) < 1:
            previous_id = ">"
            continue
        streamId, testDetails = newTestInfo[0][1][0]
        previous_id = streamId.decode()
        logging.info("Received work . Stream id {}.".format(streamId))

        if b"git_hash" in testDetails:
            build_artifacts, git_hash, run_image = extract_build_info_from_streamdata(
                testDetails
            )

            files = get_benchmark_specs(testsuites_folder)

            for test_file in files:
                redis_containers = []
                client_containers = []

                with open(test_file, "r") as stream:
                    benchmark_config, test_name = get_final_benchmark_config(
                        None, stream, ""
                    )

                    (
                        redis_configuration_parameters,
                        _,
                    ) = extract_redis_dbconfig_parameters(benchmark_config, "dbconfig")
                    for topology_spec_name in benchmark_config["redis-topologies"]:
                        try:
                            current_cpu_pos = 0
                            ceil_db_cpu_limit = extract_db_cpu_limit(
                                topologies_map, topology_spec_name
                            )
                            temporary_dir = tempfile.mkdtemp(dir=home)
                            logging.info(
                                "Using local temporary dir to persist redis build artifacts. Path: {}".format(
                                    temporary_dir
                                )
                            )
                            benchmark_tool = "redis-benchmark"
                            for build_artifact in build_artifacts:
                                buffer = testDetails[
                                    bytes("{}".format(build_artifact).encode())
                                ]
                                artifact_fname = "{}/{}".format(
                                    temporary_dir, build_artifact
                                )
                                with open(artifact_fname, "wb") as fd:
                                    fd.write(buffer)
                                    os.chmod(artifact_fname, 755)
                                # TODO: re-enable
                                # if build_artifact == "redis-server":
                                #     redis_server_path = artifact_fname

                                logging.info(
                                    "Successfully restored {} into {}".format(
                                        build_artifact, artifact_fname
                                    )
                                )
                            port = 6379
                            mnt_point = "/mnt/redis/"
                            command = generate_standalone_redis_server_args(
                                "{}redis-server".format(mnt_point),
                                port,
                                mnt_point,
                                redis_configuration_parameters,
                            )
                            command_str = " ".join(command)
                            db_cpuset_cpus, current_cpu_pos = generate_cpuset_cpus(
                                ceil_db_cpu_limit, current_cpu_pos
                            )
                            logging.info(
                                "Running redis-server on docker image {} (cpuset={}) with the following args: {}".format(
                                    run_image, db_cpuset_cpus, command_str
                                )
                            )
                            container = docker_client.containers.run(
                                image=run_image,
                                volumes={
                                    temporary_dir: {"bind": mnt_point, "mode": "rw"},
                                },
                                auto_remove=True,
                                privileged=True,
                                working_dir=mnt_point,
                                command=command_str,
                                network_mode="host",
                                detach=True,
                                cpuset_cpus=db_cpuset_cpus,
                            )
                            redis_containers.append(container)

                            full_benchmark_path = "/usr/local/bin/redis-benchmark"
                            client_mnt_point = "/mnt/client/"
                            benchmark_tool_workdir = client_mnt_point

                            # setup the benchmark
                            (
                                start_time,
                                start_time_ms,
                                start_time_str,
                            ) = get_start_time_vars()
                            local_benchmark_output_filename = (
                                get_local_run_full_filename(
                                    start_time_str,
                                    git_hash,
                                    test_name,
                                    "oss-standalone",
                                )
                            )
                            logging.info(
                                "Will store benchmark json output to local file {}".format(
                                    local_benchmark_output_filename
                                )
                            )

                            # prepare the benchmark command
                            (
                                benchmark_command,
                                benchmark_command_str,
                            ) = prepare_benchmark_parameters(
                                benchmark_config,
                                full_benchmark_path,
                                port,
                                "localhost",
                                local_benchmark_output_filename,
                                False,
                                benchmark_tool_workdir,
                                False,
                            )
                            ceil_client_cpu_limit = extract_client_cpu_limit(
                                benchmark_config
                            )
                            client_cpuset_cpus, current_cpu_pos = generate_cpuset_cpus(
                                ceil_client_cpu_limit, current_cpu_pos
                            )
                            r = redis.StrictRedis(port=6379)
                            r.ping()

                            client_container_image = extract_client_container_image(
                                benchmark_config
                            )
                            logging.info(
                                "Using docker image {} as benchmark client image (cpuset={}) with the following args: {}".format(
                                    client_container_image,
                                    client_cpuset_cpus,
                                    benchmark_command_str,
                                )
                            )
                            # run the benchmark
                            # TODO: re-enable
                            # benchmark_start_time = datetime.datetime.now()

                            client_container_stdout = docker_client.containers.run(
                                image=client_container_image,
                                volumes={
                                    temporary_dir: {
                                        "bind": client_mnt_point,
                                        "mode": "rw",
                                    },
                                },
                                auto_remove=True,
                                privileged=True,
                                working_dir=benchmark_tool_workdir,
                                command=benchmark_command_str,
                                network_mode="host",
                                detach=False,
                                cpuset_cpus=client_cpuset_cpus,
                            )

                            # TODO: re-enable
                            # benchmark_end_time = datetime.datetime.now()
                            # benchmark_duration_seconds = (
                            #     calculate_benchmark_duration_and_check(
                            #         benchmark_end_time, benchmark_start_time
                            #     )
                            # )
                            logging.info("output {}".format(client_container_stdout))
                            r.shutdown(save=False)
                            post_process_benchmark_results(
                                benchmark_tool,
                                local_benchmark_output_filename,
                                start_time_ms,
                                start_time_str,
                                client_container_stdout,
                                None,
                            )

                            with open(
                                local_benchmark_output_filename, "r"
                            ) as json_file:
                                results_dict = json.load(json_file)
                                logging.info(
                                    "Final JSON result {}".format(results_dict)
                                )

                            if args.datasink_push_results_redistimeseries:
                                logging.info("Pushing results to RedisTimeSeries.")
                                # redistimeseries_results_logic(
                                #     artifact_version,
                                #     benchmark_config,
                                #     default_metrics,
                                #     deployment_type,
                                #     exporter_timemetric_path,
                                #     results_dict,
                                #     rts,
                                #     test_name,
                                #     tf_github_branch,
                                #     tf_github_org,
                                #     tf_github_repo,
                                #     tf_triggering_env,
                                # )
                                # try:
                                #     rts.redis.sadd(testcases_setname, test_name)
                                #     rts.incrby(
                                #         tsname_project_total_success,
                                #         1,
                                #         timestamp=start_time_ms,
                                #         labels=get_project_ts_tags(
                                #             tf_github_org,
                                #             tf_github_repo,
                                #             deployment_type,
                                #             tf_triggering_env,
                                #         ),
                                #     )
                                #     #
                                # except redis.exceptions.ResponseError as e:
                                #     logging.warning(
                                #         "Error while updating secondary data structures {}. ".format(
                                #             e.__str__()
                                #         )
                                #     )
                                #     pass

                        except:
                            logging.critical(
                                "Some unexpected exception was caught "
                                "during local work. Failing test...."
                            )
                            logging.critical(sys.exc_info()[0])
                            print("-" * 60)
                            traceback.print_exc(file=sys.stdout)
                            print("-" * 60)
                        # tear-down
                        logging.info("Tearing down setup")
                        for container in redis_containers:
                            container.stop()
                        for container in client_containers:
                            if type(container) != bytes:
                                container.stop()
                        shutil.rmtree(temporary_dir, ignore_errors=True)

        else:
            logging.error("Missing commit information within received message.")
            continue


def get_benchmark_specs(testsuites_folder):
    files = pathlib.Path(testsuites_folder).glob("*.yml")
    files = [str(x) for x in files]
    logging.info(
        "Running all specified benchmarks: {}".format(" ".join([str(x) for x in files]))
    )
    return files


def extract_build_info_from_streamdata(testDetails):
    git_hash = testDetails[b"git_hash"]
    if type(git_hash) == bytes:
        git_hash = git_hash.decode()
    logging.info("Received commit hash specifier {}.".format(git_hash))
    build_artifacts_str = "redis-server"
    build_image = testDetails[b"build_image"].decode()
    run_image = build_image
    if b"run_image" in testDetails[b"run_image"]:
        run_image = testDetails[b"run_image"].decode()
    if b"build_artifacts" in testDetails:
        build_artifacts_str = testDetails[b"build_artifacts"].decode()
    build_artifacts = build_artifacts_str.split(",")
    return build_artifacts, git_hash, run_image


def generate_cpuset_cpus(ceil_db_cpu_limit, current_cpu_pos):
    previous_cpu_pos = current_cpu_pos
    current_cpu_pos = current_cpu_pos + int(ceil_db_cpu_limit)
    db_cpuset_cpus = ",".join(
        [str(x) for x in range(previous_cpu_pos, current_cpu_pos)]
    )
    return db_cpuset_cpus, current_cpu_pos


def extract_db_cpu_limit(topologies_map, topology_spec_name):
    topology_spec = topologies_map[topology_spec_name]
    db_cpu_limit = topology_spec["resources"]["requests"]["cpus"]
    ceil_db_cpu_limit = math.ceil(float(db_cpu_limit))
    return ceil_db_cpu_limit


def generate_standalone_redis_server_args(
    binary, port, dbdir, configuration_parameters=None
):
    added_params = ["port", "protected-mode", "dir"]
    # start redis-server
    command = [
        binary,
        "--protected-mode",
        "no",
        "--port",
        "{}".format(port),
        "--dir",
        dbdir,
    ]
    if configuration_parameters is not None:
        for parameter, parameter_value in configuration_parameters.items():
            if parameter not in added_params:
                command.extend(
                    [
                        "--{}".format(parameter),
                        parameter_value,
                    ]
                )
    return command



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['redis_benchmarks_specification',
 'redis_benchmarks_specification.__api__',
 'redis_benchmarks_specification.__builder__',
 'redis_benchmarks_specification.__common__',
 'redis_benchmarks_specification.__self_contained_coordinator__',
 'redis_benchmarks_specification.__setups__']

package_data = \
{'': ['*'],
 'redis_benchmarks_specification': ['setups/builders/*',
                                    'setups/platforms/*',
                                    'setups/topologies/*',
                                    'test-suites/*']}

install_requires = \
['Flask-HTTPAuth>=4.4.0,<5.0.0',
 'Flask>=2.0.1,<3.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'Werkzeug>=2.0.1,<3.0.0',
 'argparse>=1.4.0,<2.0.0',
 'black==20.8b1',
 'docker>=5.0.0,<6.0.0',
 'flask-restx>=0.5.0,<0.6.0',
 'marshmallow>=3.12.2,<4.0.0',
 'psutil>=5.8.0,<6.0.0',
 'redis>=3.5.3,<4.0.0',
 'redisbench-admin>=0.4.8,<0.5.0']

entry_points = \
{'console_scripts': ['redis-benchmarks-spec-api = '
                     'redis_benchmarks_specification.__api__.api:main',
                     'redis-benchmarks-spec-builder = '
                     'redis_benchmarks_specification.__builder__.builder:main',
                     'redis-benchmarks-spec-sc-coordinator = '
                     'redis_benchmarks_specification.__self_contained_coordinator__.self_contained_coordinator:main']}

setup_kwargs = {
    'name': 'redis-benchmarks-specification',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'filipecosta90',
    'author_email': 'filipecosta.90@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
